@@ -9,7 +9,7 @@ import logging
 import os
 from typing import TYPE_CHECKING, Any, Literal, overload
 
-from upath import registry
+from upath import UPath
 
 
 if TYPE_CHECKING:
@@ -17,37 +17,11 @@ if TYPE_CHECKING:
 
     import fsspec
     from fsspec.asyn import AsyncFileSystem
-    from upath import UPath
+    from upath.types import JoinablePathLike
 
 
-StrPath = str | os.PathLike[str]
-
 logger = logging.getLogger(__name__)
 
-loaded = False
-if not loaded:
-    import fsspec
-
-    from upathtools.filesystems.distribution_fs import DistributionFS, DistributionPath
-    from upathtools.filesystems.module_fs import ModuleFS, ModulePath
-    from upathtools.filesystems.package_fs import PackageFS, PackagePath
-    from upathtools.filesystems.python_ast_fs import AstPath, PythonAstFS
-    from upathtools.filesystems.union_fs import UnionFileSystem, UnionPath
-
-    fsspec.register_implementation("pkg", PackageFS, clobber=True)
-    fsspec.register_implementation("distribution", DistributionFS, clobber=True)
-    fsspec.register_implementation("mod", ModuleFS, clobber=True)
-    fsspec.register_implementation("ast", PythonAstFS, clobber=True)
-    fsspec.register_implementation("union", UnionFileSystem, clobber=True)
-
-    registry.register_implementation("pkg", PackagePath, clobber=True)
-    registry.register_implementation("distribution", DistributionPath, clobber=True)
-    registry.register_implementation("mod", ModulePath, clobber=True)
-    registry.register_implementation("ast", AstPath, clobber=True)
-    registry.register_implementation("union", UnionPath, clobber=True)
-
-    loaded = True
-
 
 @lru_cache(maxsize=32)
 def _get_cached_fs(protocol_or_fs: str | fsspec.AbstractFileSystem) -> AsyncFileSystem:
@@ -70,31 +44,98 @@ def _get_cached_fs(protocol_or_fs: str | fsspec.AbstractFileSystem) -> AsyncFile
 
 
 async def get_async_fs(
-    path_or_fs: StrPath | fsspec.AbstractFileSystem,
+    path_or_fs: JoinablePathLike | fsspec.AbstractFileSystem,
 ) -> AsyncFileSystem:
     """Get appropriate async filesystem for path."""
     import fsspec
-    from upath import UPath
+
+    from upathtools.helpers import to_upath
 
     if isinstance(path_or_fs, fsspec.AbstractFileSystem):
         return _get_cached_fs(path_or_fs)
 
-    path_obj = UPath(path_or_fs)
+    path_obj = to_upath(path_or_fs)
     return _get_cached_fs(path_obj.protocol)
 
 
+async def is_directory(
+    fs: AsyncFileSystem,
+    path: str,
+    *,
+    entry_type: str | None = None,
+) -> bool:
+    """Smart directory detection that trusts filesystem metadata when available.
+
+    Args:
+        fs: Async filesystem instance
+        path: Path to check
+        entry_type: Optional type info from _ls() call (e.g., "directory", "file")
+
+    Returns:
+        True if path is a directory, False otherwise
+
+    Note:
+        When entry_type is provided:
+        - "directory" -> True (no filesystem call)
+        - "file" -> False (no filesystem call)
+        - anything else -> calls fs._isdir() to check
+
+        When entry_type is None, always calls fs._isdir()
+    """
+    if entry_type == "directory":
+        return True
+    if entry_type == "file":
+        return False
+    # Uncertain or no type info - check filesystem
+    return await fs._isdir(path)
+
+
+def is_directory_sync(
+    fs: fsspec.AbstractFileSystem,
+    path: str,
+    *,
+    entry_type: str | None = None,
+) -> bool:
+    """Smart directory detection that trusts filesystem metadata when available (sync version).
+
+    Args:
+        fs: Filesystem instance
+        path: Path to check
+        entry_type: Optional type info from ls() call (e.g., "directory", "file")
+
+    Returns:
+        True if path is a directory, False otherwise
+
+    Note:
+        When entry_type is provided:
+        - "directory" -> True (no filesystem call)
+        - "file" -> False (no filesystem call)
+        - anything else -> calls fs.isdir() to check
+
+        When entry_type is None, always calls fs.isdir()
+    """
+    if entry_type == "directory":
+        return True
+    if entry_type == "file":
+        return False
+    # Uncertain or no type info - check filesystem
+    return fs.isdir(path)
+
+
 @overload
 async def read_path(
-    path: StrPath, mode: Literal["rt"] = "rt", encoding: str = ...
+    path: JoinablePathLike,
+    mode: Literal["rt"] = "rt",
+    encoding: str = ...,
 ) -> str: ...
 
 
 @overload
-async def read_path(path: StrPath, mode: Literal["rb"], encoding: str = ...) -> bytes: ...
+async def read_path(path: JoinablePathLike, mode: Literal["rb"], encoding: str = ...) -> bytes: ...
 
 
 async def read_path(
-    path: StrPath,
+    path: JoinablePathLike,
     mode: Literal["rt", "rb"] = "rt",
     encoding: str = "utf-8",
 ) -> str | bytes:
@@ -108,9 +149,9 @@ async def read_path(
     Returns:
         File content as string or bytes depending on mode
     """
-    from upath import UPath
+    from upathtools.helpers import to_upath
 
-    path_obj = UPath(path)
+    path_obj = to_upath(path)
     fs = await get_async_fs(path_obj)
     f = await fs.open_async(path_obj.path, mode=mode)
     async with f:
@@ -119,7 +160,7 @@ async def read_path(
 
 @overload
 async def read_folder(
-    path: StrPath,
+    path: JoinablePathLike,
     *,
     pattern: str = "**/*",
     recursive: bool = True,
@@ -135,7 +176,7 @@ async def read_folder(
 
 @overload
 async def read_folder(
-    path: StrPath,
+    path: JoinablePathLike,
     *,
     pattern: str = "**/*",
     recursive: bool = True,
@@ -150,7 +191,7 @@ async def read_folder(
 
 
 async def read_folder(
-    path: StrPath,
+    path: JoinablePathLike,
     *,
     pattern: str = "**/*",
     recursive: bool = True,
@@ -182,9 +223,9 @@ async def read_folder(
     Raises:
         FileNotFoundError: If base path doesn't exist
     """
-    from upath import UPath
+    from upathtools.helpers import to_upath
 
-    base_path = UPath(path)
+    base_path = to_upath(path)
     matching_files = await list_files(
         path,
         pattern=pattern,
@@ -198,7 +239,7 @@ async def read_folder(
 
     if load_parallel:
         # Process files in chunks
-        for chunk in batched(matching_files, chunk_size):
+        for chunk in batched(matching_files, chunk_size, strict=False):
             # Create tasks for this chunk
             tasks = [read_path(p, mode=mode, encoding=encoding) for p in chunk]
             # Execute chunk in parallel
@@ -206,23 +247,20 @@ async def read_folder(
                 contents: Sequence[str | bytes] = await asyncio.gather(*tasks)
                 # Map results back to relative paths
                 for file_path, content in zip(chunk, contents, strict=True):
-                    rel_path = os.path.relpath(file_path, base_path)
+                    rel_path = os.path.relpath(str(file_path), str(base_path))
                     result[rel_path] = content
             except Exception as e:  # noqa: BLE001
-                logger.warning(
-                    "Failed to read chunk starting at %s: %s",
-                    os.path.relpath(chunk[0], base_path),
-                    e,
-                )
+                msg = "Failed to read chunk starting at %s: %s"
+                logger.warning(msg, os.path.relpath(str(chunk[0]), str(base_path)), e)
     else:
         # Sequential reading
         for file_path in matching_files:
             try:
                 content = await read_path(file_path, mode=mode, encoding=encoding)
-                rel_path = os.path.relpath(file_path, base_path)
+                rel_path = os.path.relpath(str(file_path), str(base_path))
                 result[rel_path] = content
             except Exception as e:  # noqa: BLE001
-                rel_path = os.path.relpath(file_path, base_path)
+                rel_path = os.path.relpath(str(file_path), str(base_path))
                 logger.warning("Failed to read %s: %s", rel_path, e)
 
     return result
@@ -230,7 +268,7 @@ async def read_folder(
 
 @overload
 async def list_files(
-    path: StrPath,
+    path: JoinablePathLike,
     *,
     pattern: str = "**/*",
     recursive: bool = True,
@@ -243,19 +281,19 @@ async def list_files(
 
 @overload
 async def list_files(
-    path: StrPath,
+    path: JoinablePathLike,
     *,
     pattern: str = "**/*",
     recursive: bool = True,
     include_dirs: bool = False,
     exclude: list[str] | None = None,
     max_depth: int | None = None,
-    detail: Literal[True],
+    detail: Literal[True] = ...,
 ) -> list[dict[str, Any]]: ...
 
 
 async def list_files(
-    path: StrPath,
+    path: JoinablePathLike,
     *,
     pattern: str = "**/*",
     recursive: bool = True,
@@ -283,9 +321,9 @@ async def list_files(
     """
     from fnmatch import fnmatch
 
-    from upath import UPath
+    from upathtools.helpers import to_upath
 
-    base_path = UPath(path)
+    base_path = to_upath(path)
     if not base_path.exists():
         msg = f"Path does not exist: {path}"
         raise FileNotFoundError(msg)
@@ -295,14 +333,11 @@ async def list_files(
 
     # Get all matching paths
     if recursive:
-        paths = await fs._glob(
-            str(base_path / pattern), maxdepth=max_depth, detail=detail
-        )
+        glob_path = str(base_path / pattern)
+        paths = await fs._glob(glob_path, maxdepth=max_depth, detail=detail)
     else:
         paths = await fs._glob(str(base_path / pattern), detail=detail)
 
-    # Filter and collect paths
-
     # Filter and collect paths
     if detail:
         assert isinstance(paths, dict)
@@ -344,7 +379,7 @@ async def list_files(
 
 
 async def read_folder_as_text(
-    path: StrPath,
+    path: JoinablePathLike,
     *,
     pattern: str = "**/*",
     recursive: bool = True,
@@ -415,10 +450,9 @@ async def read_folder_as_text(
 
 
 if __name__ == "__main__":
-    import asyncio
     from pprint import pprint
 
-    async def main():
+    async def main() -> None:
         # Test with current directory
         files = await read_folder(
             ".",
